﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Collections;
using System.Reflection;
using System.IO;
using Common.Ajax.Utility;
using Common.Ajax.Extension;
using Common.Ajax.Service;

namespace Common.Ajax.Handlers
{
    internal class CommonPageHandlerFactory : PageHandlerFactory { }

    internal class AjaxHandlerFactory : IHttpHandlerFactory
    {
        public void ReleaseHandler(IHttpHandler handler)
        {
        }

        public IHttpHandler GetHandler(HttpContext context, string requestType, string url, string pathTranslated)
        {
            HttpRequest request = context.Request;
            if (string.Compare(request.Headers[AjaxConfig.Key], AjaxConfig.Value, true) == 0)
            {
                string methodName = request.Headers[AjaxConfig.MethodName];
                if (methodName.IsNullOrEmpty())
                {
                    Executor.EndCurrentRequest(context, "方法名称未正确指定！");
                    return null;
                }
                try
                {                    
                    CommonPageHandlerFactory ajaxPageHandler = new CommonPageHandlerFactory();
                    IHttpHandler handler = ajaxPageHandler.GetHandler(context, requestType, url, pathTranslated);
                    Page page = handler as Page;
                    if (page == null)
                    {
                        Executor.EndCurrentRequest(context, "处理程序类型必须是aspx页面！");
                        return null;
                    }
                    return GetHandler(page, methodName, context);
                }
                catch
                {
                    Executor.EndCurrentRequest(context, url + " 不存在！");
                    return null;
                }
            }
            if (url.EndsWith(".aspx", StringComparison.CurrentCultureIgnoreCase))
            {
                CommonPageHandlerFactory orgPageHandler = new CommonPageHandlerFactory();
                return orgPageHandler.GetHandler(context, requestType, url, pathTranslated);
            }
            return null;
        }

        /// <summary>
        /// 获取自定义处理程序
        /// </summary>
        /// <param name="page">处理页面</param>
        /// <param name="methodName">处理方法</param>
        /// <param name="context">当前请求</param>
        private IHttpHandler GetHandler(Page page, string methodName, HttpContext context)
        {
            CacheMethodInfo methodInfo = Executor.GetDelegateInfo(page, methodName);
            if (methodInfo == null)
            {
                Executor.EndCurrentRequest(context, "找不到指定的Ajax方法！");
                return null;
            }
            AjaxMethodAttribute attribute = methodInfo.AjaxMethodAttribute;
            if (attribute.ServerCache > 0)
            {
                object data = CacheHelper.TryGetCache(context);
                if (data != null)
                {
                    Executor.EndCurrentRequest(context, data);
                    return null;
                }
            }
            if (attribute.IsAsync)
            {
                return ASyncAjaxHandler.CreateHandler(page, methodInfo, attribute.SessionState);
            }
            return SyncAjaxHandler.CreateHandler(page, methodInfo, attribute.SessionState);
        }
    }
}