﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using Common.Ajax.Service;
using Common.Ajax.Utility;

namespace Common.Ajax.Handlers
{
    internal class ASyncAjaxHandler : IHttpAsyncHandler, IHttpHandler
    {
        private Page _page;
        private CacheMethodInfo _cacheMethodInfo;

        internal ASyncAjaxHandler(Page page, CacheMethodInfo cacheMethodInfo)
        {
            _page = page;
            _cacheMethodInfo = cacheMethodInfo;    
        }

        public IAsyncResult BeginProcessRequest(HttpContext context, AsyncCallback cb, object extraData)
        {
            /*
             * IHttpAsyncHandler异步接口提供了单位时间内线程的可用数，但真正的异步操作由调用者实现。             
             * 例如，异步读取网络数据，BeginProcessRequest执行后，该线程会立即回asp.net线程池，从而提高系统吞吐量
             * 当异步操作后，再重新取出一个线程处理。
             * 所以，下面的BeginInvoke实际没什么作用，在Asp.net，BeginInvoke调用的还是asp.net线程池的线程，只是取出一个新的线程
             * 处理任务而已。
             */
            Action<Page, HttpContext, CacheMethodInfo> action = new Action<Page, HttpContext, CacheMethodInfo>(Executor.Execute);
            IAsyncResult result = action.BeginInvoke(_page, context, _cacheMethodInfo, cb, action);
            return result;
        }

        public void EndProcessRequest(IAsyncResult result)
        {
            Action<Page, HttpContext, CacheMethodInfo> action = result.AsyncState as Action<Page, HttpContext, CacheMethodInfo>;
            action.EndInvoke(result);
        }

        public void ProcessRequest(HttpContext context)
        {
            throw new NotImplementedException();
        }

        public bool IsReusable
        {
            get { return false; }
        }

        public static ASyncAjaxHandler CreateHandler(Page page, CacheMethodInfo cacheMethodInfo, SessionState state)
        {
            switch (state)
            {
                case SessionState.ReadOnly:
                    return new ASyncAjaxSessionReadOnlyHandler(page, cacheMethodInfo);
                case SessionState.ReadWrite:
                    return new ASyncAjaxSessionHandler(page, cacheMethodInfo);                
                default:
                    return new ASyncAjaxHandler(page, cacheMethodInfo);
            }
        }
    }

    internal class ASyncAjaxSessionReadOnlyHandler : ASyncAjaxHandler, IReadOnlySessionState
    {
        internal ASyncAjaxSessionReadOnlyHandler(Page page, CacheMethodInfo cacheMethodInfo)
            : base(page, cacheMethodInfo)
        {
        }
    }

    internal class ASyncAjaxSessionHandler : ASyncAjaxHandler, IRequiresSessionState
    {
        internal ASyncAjaxSessionHandler(Page page, CacheMethodInfo cacheMethodInfo)
            : base(page, cacheMethodInfo)
        {
        }
    }    
}
