﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using Common.Ajax.Service;
using Common.Ajax.Utility;

namespace Common.Ajax.Handlers
{
    internal class SyncAjaxHandler : IHttpHandler
    {        
        private Page _page;
        private CacheMethodInfo _cacheMethodInfo;

        internal SyncAjaxHandler(Page page, CacheMethodInfo cacheMethodInfo)
        {
            _page = page;
            _cacheMethodInfo = cacheMethodInfo;
        }

        public void ProcessRequest(HttpContext context)
        {
            Executor.Execute(_page, context, _cacheMethodInfo);
        }

        public bool IsReusable
        {
            get { return false; }
        }

        public static SyncAjaxHandler CreateHandler(Page page, CacheMethodInfo cacheMethodInfo, SessionState state)
        {
            switch (state)
            {
                case SessionState.ReadOnly:
                    return new SyncAjaxSessionReadOnlyHandler(page, cacheMethodInfo);
                case SessionState.ReadWrite:
                    return new SyncAjaxSessionHandler(page, cacheMethodInfo);
                default:
                    return new SyncAjaxHandler(page, cacheMethodInfo);
            }
        }
    }

    internal class SyncAjaxSessionReadOnlyHandler : SyncAjaxHandler, IReadOnlySessionState 
    {
        internal SyncAjaxSessionReadOnlyHandler(Page page, CacheMethodInfo cacheMethodInfo)
            : base(page, cacheMethodInfo)
        {
        }
    }

    internal class SyncAjaxSessionHandler : SyncAjaxHandler, IRequiresSessionState
    {
        internal SyncAjaxSessionHandler(Page page, CacheMethodInfo cacheMethodInfo)
            : base(page, cacheMethodInfo)
        {
        }
    }    
}
