﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Caching;
using Common.Ajax.Extension;

namespace Common.Ajax.Utility
{
    internal static class CacheHelper
    {
        internal static void Insert(HttpContext context, int time, object data)
        {            
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            string key = GetCacheKey(context.Request);
            TimeSpan ts = new TimeSpan(0, 0, time);
            HttpRuntime.Cache.Insert(key, data, null, Cache.NoAbsoluteExpiration, ts);
        }

        internal static object TryGetCache(HttpContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            string key = GetCacheKey(context.Request);
            return HttpRuntime.Cache[key];
        }

        private static string GetCacheKey(HttpRequest request)
        {
            string key;
            if (string.Compare(request.HttpMethod, "GET", true) == 0)
            {
                //GET key形式：XHR|MethodName|URL
                key = AjaxConfig.Key + "|" + request.Headers["MethodName"] + "|" + request.RawUrl;
            }
            else
            {
                //POST key形式：XHR|MethodName|URL|1,2,3
                string values = string.Empty;                
                foreach (var v in request.Form)
                {
                    values += "," + v;
                }
                if (!values.IsNullOrEmpty())
                {
                    values = values.Substring(1);
                }
                key = AjaxConfig.Key + "|" + request.Headers["MethodName"] + "|" + request.RawUrl + "|" + values;
            }
            return key;
        }
    }
}
