﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;
using System.Collections;
using System.Collections.Specialized;
using Common.Ajax.Extension;

namespace Common.Ajax.Utility
{
    /// <summary>
    /// 反射帮助类
    /// </summary>
    internal static class ReflectionUtil
    {
        /// <summary>
        /// 获取通用方法委托
        /// </summary>
        /// <param name="type">实例类型</param>
        /// <param name="methodName">方法名称</param>
        /// <returns>通用委托</returns>
        public static Func<object, object[], object> GetMethodDelegate(MethodInfo methodInfo)
        {
            if (methodInfo == null)
            {
                throw new ArgumentNullException("methodInfo");
            }
            ParameterExpression instanceExp = Expression.Parameter(typeof(object), "instance");
            ParameterExpression paramExp = Expression.Parameter(typeof(object[]), "parameters");
            ParameterInfo[] paramInfos = methodInfo.GetParameters();
            List<Expression> paramExpList = new List<Expression>();
            int length = paramInfos.Length;
            for (int i = 0; i < length; i++)
            {
                BinaryExpression valueObj = Expression.ArrayIndex(paramExp, Expression.Constant(i));
                UnaryExpression valueCast = Expression.Convert(valueObj, paramInfos[i].ParameterType);
                paramExpList.Add(valueCast);
            }
            UnaryExpression instanceCast = methodInfo.IsStatic ? null : Expression.Convert(instanceExp, methodInfo.ReflectedType);
            MethodCallExpression methodCall = Expression.Call(instanceCast, methodInfo, paramExpList);               
            if (methodCall.Type == typeof(void))
            {
                Expression<Action<object, object[]>> lambda = Expression.Lambda<Action<object, object[]>>(methodCall, instanceExp, paramExp);
                Action<object, object[]> action = lambda.Compile();
                return (instance, parameters) =>
                {
                    action(instance, parameters);
                    return null;
                };
            }
            else
            {
                UnaryExpression castMethodCall = Expression.Convert(methodCall, typeof(object));
                Expression<Func<object, object[], object>> lambda = Expression.Lambda<Func<object, object[], object>>(castMethodCall, instanceExp, paramExp);
                return lambda.Compile();
            }
        }

        /// <summary>
        /// 获取构造函数委托
        /// </summary>
        /// <param name="type">实例类型</param>
        /// <returns></returns>
        public static Func<object> GetConstructorDelegate(Type type)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }
            ConstructorInfo ci = type.GetConstructor(Type.EmptyTypes);
            if (ci == null)
            {
                throw new MissingMemberException("类型必须有一个无参public构造函数！");
            }
            NewExpression newExp = Expression.New(type);
            Expression<Func<object>>  lambda = Expression.Lambda<Func<object>>(newExp);
            return lambda.Compile();
        }
    }
}